@@ -19,7 +19,11 @@
 
 #ifndef __ACCESSX_APPLET_H__
 #define __ACCESSX_APPLET_H__
-#include <gnome.h>
+
+#include <gtk/gtkiconfactory.h>
+#include <gtk/gtktooltips.h>
+#include <gtk/gtkwidget.h>
+
 #include <panel-applet.h>
 
 #define ACCESSX_APPLET          "ax-applet"
@@ -69,7 +73,6 @@ typedef struct
 	GtkWidget        *hyper_indicator;
 	GtkWidget        *super_indicator;
 	GtkWidget        *alt_graph_indicator;
-	GtkWidget        *about_dialog;
 	GtkTooltips      *tooltips;
 	PanelAppletOrient orient;
 	GtkIconFactory   *icon_factory;
