@@ -26,7 +26,11 @@
 #include <gtk/gtkdialog.h>
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
+#ifdef HAVE_GST10
+#include <gst/interfaces/mixer.h>
+#else
 #include <gst/mixer/mixer.h>
+#endif
 
 G_BEGIN_DECLS
 
@@ -40,7 +44,7 @@ G_BEGIN_DECLS
 			    GnomeVolumeAppletPreferencesClass))
 #define GNOME_VOLUME_APPLET_IS_PREFERENCES(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GNOME_VOLUME_APPLET_TYPE_PREFERENCES))
-#define GNOME_VOLUME_APPLET_IS_PREFERENCES_CLASS(obj) \
+#define GNOME_VOLUME_APPLET_IS_PREFERENCES_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE ((klass), GNOME_VOLUME_APPLET_TYPE_PREFERENCES))
 
 typedef struct _GnomeVolumeAppletPreferences {
@@ -52,6 +56,9 @@ typedef struct _GnomeVolumeAppletPreferences {
   /* current element that we're working on */
   GstMixer *mixer;
 
+  /* is the track list currently locked */
+  gboolean track_lock;
+
   /* for gconf */
   PanelApplet *applet;
 
@@ -67,10 +74,10 @@ GType	gnome_volume_applet_preferences_get_type (void);
 GtkWidget *gnome_volume_applet_preferences_new	(PanelApplet *applet,
 						 GList       *elements,
 						 GstMixer    *mixer,
-						 GstMixerTrack *track);
+						 GList       *track);
 void	gnome_volume_applet_preferences_change	(GnomeVolumeAppletPreferences *prefs,
 						 GstMixer    *mixer,
-						 GstMixerTrack *track);
+						 GList       *tracks);
 
 G_END_DECLS
 
